/*
 *   Copyright 2011 Marco Martin <mart@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2 or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Library General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "slcservice.h"
#include "slcjob.h"

namespace SLC {

SlcService::SlcService(const QString &action, ShareLikeConnectEngine *parent, const QVariantList &args)
    : Plasma::Service(parent, args),
      m_slcEngine(parent)
{
    setName("slcservice");

    if (action == "Share") {
        m_action = Provider::Share;
    } else if (action == "Like") {
        m_action = Provider::Like;
    //Connect
    } else {
        m_action = Provider::Connect;
    }
}

Plasma::ServiceJob *SlcService::createJob(const QString &operation,
                                          QMap<QString, QVariant> &parameters)
{
    if (!m_slcEngine) {
        return 0;
    }
    const QString action = parameters.value("ActionName").toString();

    Provider *provider = m_slcEngine.data()->providers().value(action);

    return new SlcJob(m_action, provider, operation, m_slcEngine.data()->content(), parameters, this);
}

}

#include "slcservice.moc"
